@@ -5,6 +5,8 @@
 #include "codegen_shared.h"
 #include "julia_assert.h"
 
+#include <llvm/IR/IntrinsicInst.h>
+
 using namespace llvm;
 using namespace jl_alloc;
 
@@ -211,7 +213,8 @@ void jl_alloc::runEscapeAnalysis(llvm::Instruction *I, EscapeAnalysisRequiredArg
                 assert(use->get() == I);
                 return true;
             }
-            if (required.pass.write_barrier_func == callee)
+            if (required.pass.write_barrier_func == callee ||
+                required.pass.write_barrier_binding_func == callee)
                 return true;
             auto opno = use->getOperandNo();
             // Uses in `jl_roots` operand bundle are not counted as escaping, everything else is.
