@@ -1,11 +1,13 @@
 // This file is a part of Julia. License is MIT: https://julialang.org/license
 
 #include "llvm-version.h"
+#include "passes.h"
 
 #include <llvm-c/Core.h>
 #include <llvm-c/Types.h>
 
 #include <llvm/ADT/DepthFirstIterator.h>
+#include <llvm/ADT/Statistic.h>
 #include <llvm/Analysis/CFG.h>
 #include <llvm/IR/BasicBlock.h>
 #include <llvm/IR/Constants.h>
@@ -15,6 +17,7 @@
 #include <llvm/IR/Module.h>
 #include <llvm/IR/Value.h>
 #include <llvm/IR/LegacyPassManager.h>
+#include <llvm/IR/Verifier.h>
 #include <llvm/Pass.h>
 #include <llvm/Support/Debug.h>
 #include <llvm/Transforms/Utils/BasicBlockUtils.h>
@@ -22,9 +25,12 @@
 #include "julia.h"
 #include "julia_assert.h"
 #include "codegen_shared.h"
+#include <map>
 
 #define DEBUG_TYPE "lower_handlers"
 #undef DEBUG
+STATISTIC(MaxExceptionHandlerDepth, "Maximum nesting of exception handlers");
+STATISTIC(ExceptionHandlerBuffers, "Number of exception handler buffers inserted");
 
 using namespace llvm;
 
@@ -69,23 +75,8 @@ using namespace llvm;
  * handler structures to tell LLVM that it is free to re-use the stack slot
  * while the handler is not being used.
  */
-struct LowerExcHandlers : public FunctionPass {
-    static char ID;
-    LowerExcHandlers() : FunctionPass(ID)
-    {}
-
-private:
-    Function *except_enter_func;
-    Function *leave_func;
-    Function *jlenter_func;
-    Function *setjmp_func;
-    Function *lifetime_start;
-    Function *lifetime_end;
-
-    bool doInitialization(Module &M) override;
-    bool runOnFunction(Function &F) override;
-};
 
+namespace {
 /*
  * If the module doesn't have declarations for the jl_enter_handler and setjmp
  * functions, insert them.
@@ -114,24 +105,19 @@ static void ensure_enter_function(Module &M)
     }
 }
 
-bool LowerExcHandlers::doInitialization(Module &M) {
-    except_enter_func = M.getFunction("julia.except_enter");
-    if (!except_enter_func)
-        return false;
-    ensure_enter_function(M);
-    leave_func = M.getFunction(XSTR(jl_pop_handler));
-    jlenter_func = M.getFunction(XSTR(jl_enter_handler));
-    setjmp_func = M.getFunction(jl_setjmp_name);
-
-    auto T_pint8 = Type::getInt8PtrTy(M.getContext(), 0);
-    lifetime_start = Intrinsic::getDeclaration(&M, Intrinsic::lifetime_start, { T_pint8 });
-    lifetime_end = Intrinsic::getDeclaration(&M, Intrinsic::lifetime_end, { T_pint8 });
-    return true;
-}
-
-bool LowerExcHandlers::runOnFunction(Function &F) {
+static bool lowerExcHandlers(Function &F) {
+    Module &M = *F.getParent();
+    Function *except_enter_func = M.getFunction("julia.except_enter");
     if (!except_enter_func)
         return false; // No EH frames in this module
+    ensure_enter_function(M);
+    Function *leave_func = M.getFunction(XSTR(jl_pop_handler));
+    Function *jlenter_func = M.getFunction(XSTR(jl_enter_handler));
+    Function *setjmp_func = M.getFunction(jl_setjmp_name);
+
+    auto T_pint8 = Type::getInt8PtrTy(M.getContext(), 0);
+    Function *lifetime_start = Intrinsic::getDeclaration(&M, Intrinsic::lifetime_start, { T_pint8 });
+    Function *lifetime_end = Intrinsic::getDeclaration(&M, Intrinsic::lifetime_end, { T_pint8 });
 
     /* Step 1: EH Depth Numbering */
     std::map<llvm::CallInst *, int> EnterDepth;
@@ -174,6 +160,8 @@ bool LowerExcHandlers::runOnFunction(Function &F) {
         /* Remember the depth at the BB boundary */
         ExitDepth[BB] = Depth;
     }
+    MaxExceptionHandlerDepth.updateMax(MaxDepth);
+    ExceptionHandlerBuffers += MaxDepth;
 
     /* Step 2: EH Frame lowering */
     // Allocate stack space for each handler. We allocate these as separate
@@ -184,17 +172,24 @@ bool LowerExcHandlers::runOnFunction(Function &F) {
     Value *handler_sz64 = ConstantInt::get(Type::getInt64Ty(F.getContext()),
                                            sizeof(jl_handler_t));
     Instruction *firstInst = &F.getEntryBlock().front();
-    std::vector<AllocaInst *> buffs;
+    std::vector<Instruction *> buffs;
+    unsigned allocaAddressSpace = F.getParent()->getDataLayout().getAllocaAddrSpace();
     for (int i = 0; i < MaxDepth; ++i) {
-        auto *buff = new AllocaInst(Type::getInt8Ty(F.getContext()), 0,
+        auto *buff = new AllocaInst(Type::getInt8Ty(F.getContext()), allocaAddressSpace,
                 handler_sz, Align(16), "", firstInst);
-        buffs.push_back(buff);
+        if (allocaAddressSpace) {
+            AddrSpaceCastInst *buff_casted = new AddrSpaceCastInst(buff, Type::getInt8PtrTy(F.getContext(), AddressSpace::Generic));
+            buff_casted->insertAfter(buff);
+            buffs.push_back(buff_casted);
+        } else {
+            buffs.push_back(buff);
+        }
     }
 
     // Lower enter funcs
     for (auto it : EnterDepth) {
         assert(it.second >= 0);
-        AllocaInst *buff = buffs[it.second];
+        Instruction *buff = buffs[it.second];
         CallInst *enter = it.first;
         auto new_enter = CallInst::Create(jlenter_func, buff, "", enter);
         Value *lifetime_args[] = {
@@ -236,14 +231,42 @@ bool LowerExcHandlers::runOnFunction(Function &F) {
     return true;
 }
 
-char LowerExcHandlers::ID = 0;
-static RegisterPass<LowerExcHandlers> X("LowerExcHandlers", "Lower Julia Exception Handlers",
+} // anonymous namespace
+
+PreservedAnalyses LowerExcHandlers::run(Function &F, FunctionAnalysisManager &AM)
+{
+    bool modified = lowerExcHandlers(F);
+#ifdef JL_VERIFY_PASSES
+    assert(!verifyFunction(F, &errs()));
+#endif
+    if (modified) {
+        return PreservedAnalyses::allInSet<CFGAnalyses>();
+    }
+    return PreservedAnalyses::all();
+}
+
+
+struct LowerExcHandlersLegacy : public FunctionPass {
+    static char ID;
+    LowerExcHandlersLegacy() : FunctionPass(ID)
+    {}
+    bool runOnFunction(Function &F) {
+        bool modified = lowerExcHandlers(F);
+#ifdef JL_VERIFY_PASSES
+        assert(!verifyFunction(F, &errs()));
+#endif
+        return modified;
+    }
+};
+
+char LowerExcHandlersLegacy::ID = 0;
+static RegisterPass<LowerExcHandlersLegacy> X("LowerExcHandlers", "Lower Julia Exception Handlers",
                                          false /* Only looks at CFG */,
                                          false /* Analysis Pass */);
 
 Pass *createLowerExcHandlersPass()
 {
-    return new LowerExcHandlers();
+    return new LowerExcHandlersLegacy();
 }
 
 extern "C" JL_DLLEXPORT void LLVMExtraAddLowerExcHandlersPass_impl(LLVMPassManagerRef PM)
