@@ -1,8 +1,7 @@
 // This file is a part of Julia. License is MIT: https://julialang.org/license
 
 #include "llvm-version.h"
-
-#define DEBUG_TYPE "lower_simd_loop"
+#include "passes.h"
 
 // This file defines a LLVM pass that:
 // 1. Set's loop information in form of metadata
@@ -19,16 +18,29 @@
 #include <llvm-c/Core.h>
 #include <llvm-c/Types.h>
 
+#include <llvm/ADT/Statistic.h>
 #include <llvm/Analysis/LoopPass.h>
 #include <llvm/IR/LegacyPassManager.h>
 #include <llvm/IR/Instructions.h>
-#include <llvm/IR/LLVMContext.h>
 #include <llvm/IR/Metadata.h>
+#include <llvm/IR/Verifier.h>
 #include <llvm/Support/Debug.h>
 
 #include "julia_assert.h"
 
-namespace llvm {
+#define DEBUG_TYPE "lower_simd_loop"
+
+using namespace llvm;
+
+STATISTIC(TotalMarkedLoops, "Total number of loops marked with simdloop");
+STATISTIC(IVDepLoops, "Number of loops with no loop-carried dependencies");
+STATISTIC(SimdLoops, "Number of loops with SIMD instructions");
+STATISTIC(IVDepInstructions, "Number of instructions marked ivdep");
+STATISTIC(ReductionChains, "Number of reduction chains folded");
+STATISTIC(ReductionChainLength, "Total sum of instructions folded from reduction chain");
+STATISTIC(MaxChainLength, "Max length of reduction chain");
+STATISTIC(AddChains, "Addition reduction chains");
+STATISTIC(MulChains, "Multiply reduction chains");
 
 namespace {
 
@@ -99,10 +111,23 @@ static void enableUnsafeAlgebraIfReduction(PHINode *Phi, Loop *L)
         }
         chain.push_back(J);
     }
+    switch (opcode) {
+        case Instruction::FAdd:
+            ++AddChains;
+            break;
+        case Instruction::FMul:
+            ++MulChains;
+            break;
+    }
+    ++ReductionChains;
+    int length = 0;
     for (chainVector::const_iterator K=chain.begin(); K!=chain.end(); ++K) {
         LLVM_DEBUG(dbgs() << "LSL: marking " << **K << "\n");
         (*K)->setFast(true);
+        ++length;
     }
+    ReductionChainLength += length;
+    MaxChainLength.updateMax(length);
 }
 
 static bool markLoopInfo(Module &M, Function *marker, function_ref<LoopInfo &(Function &)> GetLI)
@@ -110,6 +135,7 @@ static bool markLoopInfo(Module &M, Function *marker, function_ref<LoopInfo &(Fu
     bool Changed = false;
     std::vector<Instruction*> ToDelete;
     for (User *U : marker->users()) {
+        ++TotalMarkedLoops;
         Instruction *I = cast<Instruction>(U);
         ToDelete.push_back(I);
 
@@ -176,10 +202,12 @@ static bool markLoopInfo(Module &M, Function *marker, function_ref<LoopInfo &(Fu
         // If ivdep is true we assume that there is no memory dependency between loop iterations
         // This is a fairly strong assumption and does often not hold true for generic code.
         if (ivdep) {
+            ++IVDepLoops;
             // Mark memory references so that Loop::isAnnotatedParallel will return true for this loop.
             for (BasicBlock *BB : L->blocks()) {
                for (Instruction &I : *BB) {
                    if (I.mayReadOrWriteMemory()) {
+                       ++IVDepInstructions;
                        I.setMetadata(LLVMContext::MD_mem_parallel_loop_access, m);
                    }
                }
@@ -188,6 +216,7 @@ static bool markLoopInfo(Module &M, Function *marker, function_ref<LoopInfo &(Fu
         }
 
         if (simd) {
+            ++SimdLoops;
             // Mark floating-point reductions as okay to reassociate/commute.
             for (BasicBlock::iterator I = Lh->begin(), E = Lh->end(); I != E; ++I) {
                 if (PHINode *Phi = dyn_cast<PHINode>(I))
@@ -203,7 +232,9 @@ static bool markLoopInfo(Module &M, Function *marker, function_ref<LoopInfo &(Fu
     for (Instruction *I : ToDelete)
         I->deleteValue();
     marker->eraseFromParent();
-
+#ifdef JL_VERIFY_PASSES
+    assert(!verifyModule(M, &errs()));
+#endif
     return Changed;
 }
 
@@ -213,9 +244,6 @@ static bool markLoopInfo(Module &M, Function *marker, function_ref<LoopInfo &(Fu
 /// This pass should run after reduction variables have been converted to phi nodes,
 /// otherwise floating-point reductions might not be recognized as such and
 /// prevent SIMDization.
-struct LowerSIMDLoop : PassInfoMixin<LowerSIMDLoop> {
-    PreservedAnalyses run(Module &M, ModuleAnalysisManager &AM);
-};
 
 
 PreservedAnalyses LowerSIMDLoop::run(Module &M, ModuleAnalysisManager &AM)
@@ -232,7 +260,11 @@ PreservedAnalyses LowerSIMDLoop::run(Module &M, ModuleAnalysisManager &AM)
         return FAM.getResult<LoopAnalysis>(F);
     };
 
-    markLoopInfo(M, loopinfo_marker, GetLI);
+    if (markLoopInfo(M, loopinfo_marker, GetLI)) {
+        auto preserved = PreservedAnalyses::allInSet<CFGAnalyses>();
+        preserved.preserve<LoopAnalysis>();
+        return preserved;
+    }
 
     return PreservedAnalyses::all();
 }
@@ -288,5 +320,3 @@ extern "C" JL_DLLEXPORT void LLVMExtraAddLowerSimdLoopPass_impl(LLVMPassManagerR
 {
     unwrap(PM)->add(createLowerSimdLoopPass());
 }
-
-} // namespace llvm
