@@ -28,10 +28,8 @@ extern "C" {
 
 DLLEXPORT const char *MakeIdentityFunction(jl_value_t* jl_AnyTy) {
     LLVMContext Ctx;
-    PointerType *AnyTy = PointerType::get(StructType::get(Ctx), 0);
-    // FIXME: get AnyTy via jl_type_to_llvm(Ctx, jl_AnyTy)
-
-    Type *TrackedTy = PointerType::get(AnyTy->getElementType(), AddressSpace::Tracked);
+    // FIXME: get TrackedTy via jl_type_to_llvm(Ctx, jl_AnyTy)
+    Type *TrackedTy = PointerType::get(StructType::get(Ctx), AddressSpace::Tracked);
     Module *M = new llvm::Module("shadow", Ctx);
     Function *F = Function::Create(
         FunctionType::get(
