@@ -79,6 +79,7 @@
 
 (define (caddddr x) (car (cdr (cdr (cdr (cdr x))))))
 (define (cdddddr x) (cdr (cdr (cdr (cdr (cdr x))))))
+(define (cadddddr x) (car (cdddddr x)))
 
 (define (table.clone t)
   (let ((nt (table)))
@@ -103,3 +104,11 @@
           (begin (put! tbl (car xs) i)
                  (loop (cdr xs) (+ i 1)))))
     tbl))
+
+;; keep at most the first element matching a given predicate
+(define (keep-first pred lst)
+  (cond ((null? lst) lst)
+        ((pred (car lst))
+         (cons (car lst) (filter (lambda (x) (not (pred x))) (cdr lst))))
+        (else
+         (cons (car lst) (keep-first pred (cdr lst))))))
